#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"


#define LED_DDR  DDRB						// Hardwarekonfiguration
#define LED_PORT PORTB
#define LED_PIN  PB1

#define ADDR_TASTMAT 0x42
#define ADDR_LEDMAT  0x44

unsigned char clicks = 0;

/* -------------------------------------------------------------------------- */
ISR(INT1_vect)
{
	// Interruptroutine wird angesprungen, wenn eine Taste gerueckt 
	// worden ist; hier wird erstmal nur die Board-LED getoggelt, 
	// man koente auch sinnvolleres machen (z.B. Tastencode nur 
	// abfragen, wenn dieser Interrupt gefeuert hat...)
	LED_PORT ^= (1 << LED_PIN);
}

/* -------------------------------------------------------------------------- */
void long_delay(uint16_t ms) {				// lange Pause
    for (; ms>0; ms--) _delay_ms(1);
}


/* -------------------------------------------------------------------------- */
unsigned char keyboard_write(unsigned char val)
{
	if (i2c_start(ADDR_TASTMAT+I2C_WRITE)) return 1;
	i2c_write(val);
	i2c_stop();
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char keyboard_read(void)
{
	unsigned char val;
	
	if (i2c_start(ADDR_TASTMAT+I2C_READ)) return 0;
	val = i2c_readNak();
	i2c_stop();
	return val;
}


/* -------------------------------------------------------------------------- */
unsigned char keyboard_init(void)
{
	// Zeilen auf High, Spalten auf Low...
	// wird eine Taste gedrueckt, aendert sich der Pegel an einer der
	// Spalten und es wird ein Interrupt ausgeloest
	if (keyboard_write(0x0F)) return 1;
	return 0;
}


/* -------------------------------------------------------------------------- */
unsigned char keyboard_scan(void)
{
	unsigned char row, col, key;
	
	row = keyboard_read();				
	if (row < 0x0F) {					// einer der Zeilen ist durchgeschalten
		keyboard_write(0xF0);			// Spalten auslesen
		col = keyboard_read();			// ...
		key = col+row;					// Keycode berechnen
		keyboard_init();				// Ausgangszustand (Zeilen ueberwachen)
		long_delay(90);					// eine Weile warten wg. Tastenprellen
		row = keyboard_read();			// und das Gleiche nochmal...
		if (row < 0x0F) {				// ... wegen Tastenprellen
			keyboard_write(0xF0);		// ...
			col = keyboard_read();
			keyboard_init();
		} else {
			return 0;	
		}
		if (key == (col+row)) {			// nochmal die gleiche Taste? 
			switch (key)				// Tastencode in was "lesbares" umsetzen
				{
					case 238 : return 1;	
					case 237 : return 2;	
					case 235 : return 3;	
					case 231 : return 4;	
					case 222 : return 5;	
					case 221 : return 6;	
					case 219 : return 7;	
					case 215 : return 8;	
					case 190 : return 9;	
					case 189 : return 10;	
					case 187 : return 11;	
					case 183 : return 12;	
					case 126 : return 13;	
					case 125 : return 14;	
					case 123 : return 15;	
					case 119 : return 16;	
				}
			return key;					
		} else {						
			return 0;					
		}		
	}
	return 0;	
}

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
int main(void)
{
	char s[10];
	unsigned char key, old_key;

	LED_DDR   = (1 << LED_PIN);				// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);			// LED ausschalten
	
	MCUCR |= (1<<ISC10) | (1<<ISC11);		// steigende Flanke (L->H) fuer INT1
	GICR  |= (1<<INT1);          			// externer Interrupt an INT1

	sei(); 									// Interrupts einschalten
	
	i2c_init();								// TWI initialisieren

	// LCD initialisieren
	lcd_init();

	// Spalten auf Low, Zeilen auf High,
	// wenn spaeter Taste gedrueckt, dann ist eine der Zeilen
	// auf Low
	if (keyboard_write(0x0F)) return 1;

	// LCD-Anzeige loeschen
	lcd_command(LCD_CLEAR);
	
	// Hintergrundbeleuchtung an
	lcd_backlight(ON);

	// Text auf LCD ausgeben
	lcd_printlc(1,1,(unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2,1,(unsigned char *)"Uwe Berger; 2008");

	// Text ca. 1s stehen lassen und Anzeige loeschen	
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);

	lcd_printlc(1,1,(unsigned char *)"Keycode: ");
	lcd_printlc(2,1,(unsigned char *)"Klicks.: ");
	
	while (1) {								
		// Tastatur abfragen
		key = keyboard_scan();
		if (key != old_key) {
			itoa(key, s, 10);
			lcd_printlc(1,10,(unsigned char *)"   ");
			lcd_printlr(1, 10, (unsigned char *)s);
			old_key = key;
			clicks++;
			itoa(clicks, s, 10);
			lcd_printlr(2, 10, (unsigned char *)s);
		}
	}
	return 0;
}
